#include "processing.h"

#include "runtime/threading.h"

#define MAP_SIZE 2048
typedef struct {
    vy_file_id fids[MAP_SIZE];
    vy_uid uids[MAP_SIZE];

    unsigned int used_slots;
} vy_uid_map;

static vy_uid_map _map;
static vy_mutex *_guard;

void vyInitUIDMap(void) {
    _guard = vyCreateMutex();
}

vy_uid vyLookupUID(vy_file_id fid) {
    vyLockMutex(_guard);
    unsigned int i = 0;
    vy_uid result  = VY_INVALID_UID;
    while (i < MAP_SIZE) {
        unsigned int slot = (fid + i) % MAP_SIZE;
        if (_map.fids[slot] == fid) {
            _map.fids[slot] = fid;
            result          = _map.uids[slot];
            break;
        } else if (_map.fids[slot] == VY_INVALID_FILE_ID) {
            break;
        }
    }
    vyUnlockMutex(_guard);
    return result;
}

void vyAddUIDMapping(vy_file_id fid, vy_uid uid) {
    vyLockMutex(_guard);
    float fill_rate = (float)_map.used_slots / MAP_SIZE;
    if (fill_rate >= .5f) {
        vyLog("ASSETC", "UID map is above 50% filled.");
    }
    unsigned int i = 0;
    while (i < MAP_SIZE) {
        unsigned int slot = (fid + i) % MAP_SIZE;
        if (_map.fids[slot] == 0) {
            _map.fids[slot] = fid;
            _map.uids[slot] = uid;
            break;
        }
    }
    if (i == MAP_SIZE) {
        vyReportError("ASSETC", "Failed to insert entry into UID map.");
    }
    vyUnlockMutex(_guard);
}